<?php
/*
  Plugin Name: 	WooCommerce AirPay Payment Gateway
  Plugin URI:		https://theeravat.com/product/woocommerce-airpay-payment-gateway/
  Description: 	Provides WooCommerce with AirPay Payment Gateway.Support woocommerce 2.0.0 and above.Support AirPay payment version 1.3.3 (Document Updated: 04 May 2016)
  Version:			1.0	
  Author:			Theeravat Suensilpong
  Author URI:		https://theeravat.com
  License:			commercial
  WC require at least: 2.0.0
  WC tested up to: 3.3.4
 */

add_action('plugins_loaded', 'init_airpay_gateway_class');

function init_airpay_gateway_class() {

	//Check class WC_Payment_Gateway is exists
	if (class_exists("WC_Payment_Gateway")) {

		class WC_Gateway_Airpay extends WC_Payment_Gateway {

            /** @var string Access key for ... */
            var $access_key;

            /**
             * Constructor for the gateway.
             *
             * @access public
             * @return void
             */
            public function __construct() {
                global $woocommerce;

                $this->id = 'airpay';
                $this->method_title = __('AirPay', 'woocommerce');
                $this->icon = apply_filters('woocommerce_airpay_checkout_icon', plugin_dir_url(__FILE__) . 'images/logo-airpay_74_25.png');
                $this->has_fields = false;
					 $this->airpay_get_landing_url = 'https://api.airpay.in.th/pay/get_landing_url';
					 $this->airpay_get_order_status = 'https://api.airpay.in.th/pay/get_order_status';
                $this->notify_url = str_replace('https:', 'http:', add_query_arg('wc-api', 'WC_Gateway_Airpay', home_url('/')));

                // Load the form fields.
                $this->init_form_fields();

                // Load the settings.
                $this->init_settings();

                // Define user set variables
                // Check compatibility
						$this->app_id = $this->get_option('app_id'); //App ID
						$this->app_key = $this->get_option('app_key'); //App Key
						$this->title = $this->get_option('title');
						$this->description = $this->get_option('description');

						// Save options
						add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

                // Actions
                add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
                add_action('woocommerce_thankyou_' . $this->id, array($this, 'thankyou_page'));

                // Payment listener/API hook
                add_action('woocommerce_api_wc_gateway_' . $this->id, array($this, 'airpay_notification'));
            }

            /**
             * airpay_notification function.
             *
             * @access public
             * @return void
             */
            public function airpay_notification() {
					global $woocommerce;

					//Get data from Airpay
					//Check it
					$order_id = $_GET['order_id'];
					$order = wc_get_order($order_id);
					wp_redirect($this->get_return_url($order));
					exit;
            }

            /**
             * Initialise Gateway Settings Form Fields
             *
             * @access public
             * @return void
             */
            public function init_form_fields() {
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => __('Enable/Disable', 'woocommerce'),
                        'type' => 'checkbox',
                        'label' => __('Enable AirPay Checkout', 'woocommerce'),
                        'default' => 'no'
                    ),
                    'app_id' => array(
                        'title' => __('Merchant ID/APP ID', 'woocommerce'),
                        'type' => 'text',
                        'css' => 'width:300px',
                        'description' => __('AirPay Merchant ID/APP ID', 'woocommerce'),
                        'default' => '',
                        'desc_tip' => true,
                    ),
                    'app_key' => array(
                        'title' => __('APP Key', 'woocommerce'),
                        'type' => 'text',
                        'css' => 'width:300px',
                        'description' => __('APP Key', 'woocommerce'),
                        'default' => '',
                        'desc_tip' => true,
                    ),
                    'title' => array(
                        'title' => __('Title', 'woocommerce'),
                        'type' => 'text',
                        'css' => 'width:300px',
                        'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                        'default' => __('AirPay', 'woocommerce'),
                        'desc_tip' => true,
                    ),
                    'description' => array(
                        'title' => __('Description', 'woocommerce'),
                        'type' => 'textarea',
                        'default' => __('Pay securely with AirPay.', 'woocommerce'),
                        'description' => __('This controls the description which the user sees during checkout.', 'woocommerce'),
                    ),
                );
            }

			/**
			 * Admin Panel Options
			 * - Options for bits like 'title' and availability on a country-by-country basis
			 *
			 * @access public
			 * @return void
			 */
			public function admin_options() {
				?>
				<h3><?php _e('AirPay', 'woocommerce'); ?></h3>

				<table class="form-table">
				<?php $this->generate_settings_html(); ?>
				</table>
				<?php
			}

			/**
			* Process the payment and return the result
			*
			* @access public
			* @param int $order_id
			* @return array
			*/
			public function process_payment($order_id) {
				global $woocommerce;

				$order = new WC_Order($order_id);

				return array(
					'result' => 'success',
					'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))))
				);
			}

			/**
			* Output for the order received page.
			*
			* @access public
			* @return void
			*/
			function receipt_page($order_id) {
				global $woocommerce;

				$order = new WC_Order($order_id);


				$app_id = $this->app_id;
				$app_key = $this->app_key;
				$data['app_id'] = $app_id;
				$data['app_key'] = $app_key;
				$data['order_id'] = $order_id;
				$data['return_url'] = '';
				$airpay_get_landing_url = $this->airpay_get_landing_url;
				$qr_url = plugin_dir_url(__FILE__) . "images/url-qr.php";

				//check client is desktop or mobile
				if( wp_is_mobile()){
					//get airpay landing page url
					$data['source'] = 'web';
					$data['return_url'] = $this->notify_url."&order_id=$order_id";
					$result = wp_remote_post($airpay_get_landing_url,array('body'=>$data));
					$result_obj = json_decode($result['body']);
					//if client is mobile -> redirect to airpay landing page url
               wp_redirect($result_obj->url);
               exit;
				}else{
					//get airpay landing page url
					$data['source'] = 'qr';
					$result = wp_remote_post($airpay_get_landing_url,array('body'=>$data));
					$result_obj = json_decode($result['body']);
					//if client is desktop -> show QR Code
					echo '<p>' . __('Thank you for your order, please scan QR Code below to pay with Airpay.', 'woocommerce') . '</p>';
					?>
					<img src="<?=$qr_url?>?url=<?=urlencode($result_obj->url)?>">
					<?php
					add_action('wp_footer','receipt_page_javascript');
				}
				curl_close($ch);
			}

			public function thankyou_page($order_id){
				global $woocommerce;

				$data['app_id'] = $this->app_id;
				$data['app_key'] = $this->app_key;
				$data['order_id'] = $order_id;
				$order = wc_get_order($order_id);
				if($order->get_status() == 'pending'){
					//check AirPay Order status, if customer choose Counter Service -> tell them to go to 7eleven
					$airpay_get_order_status = $this->airpay_get_order_status;
					$result = wp_remote_post($airpay_get_order_status,array('body'=>$data));
					$result_obj = json_decode($result['body']);
						//if not Counter Service -> show qr code
						$data['return_url'] = '';
						$airpay_get_landing_url = $this->airpay_get_landing_url;
						$qr_url = plugin_dir_url(__FILE__) . "images/url-qr.php";
						//check client is desktop or mobile
						if( wp_is_mobile()){
							//get airpay landing page url
							$data['source'] = 'web';
							$data['return_url'] = $this->notify_url."&order_id=$order_id";
							$result = wp_remote_post($airpay_get_landing_url,array('body'=>$data));
							$result_obj = json_decode($result['body']);
							//if client is mobile -> redirect to airpay landing page url
							//wp_redirect($result_obj->url);
							//exit;
							echo '<p>' . __('Thank you for your order. Please click the button below to pay with AirPay.','woocommerce') ."<br>
							<button onclick=\"window.location='$result_obj->url'\">Pay with AirPay</button></p>";
						}else{
							//get airpay landing page url
							$data['source'] = 'qr';
							$result = wp_remote_post($airpay_get_landing_url,array('body'=>$data));
							$result_obj = json_decode($result['body']);
							//if client is desktop -> show QR Code
							echo '<p>' . __('Thank you for your order, please scan QR Code below to pay with Airpay.', 'woocommerce') . '</p>';
							?>
							<img src="<?=$qr_url?>?url=<?=urlencode($result_obj->url)?>">
							<?php
							//javascript to check order status.. loop every 15 seconds
						}
						curl_close($ch);
				}else if($order->get_status() == 'cancelled'){
					echo '<p>' . __('You have cancelled the order.','woocommerce') . '</p>';
				}else if($order->get_status() == 'failed'){
					echo '<p>' . __('AirPay Payment Failed.','woocommerce') . '</p>';
				}
			}

		}

		/*
		* register rest route
		*/
		add_action('rest_api_init',function(){
			$version = '1';
			$namespace = 'wc-airpay/v' . $version;
			register_rest_route($namespace,'/get_order_details/', array(
				'methods' => 'POST',
				'callback' => 'wc_airpay_get_order_details',
				'args' => array(
					'app_key' => array(
						'required' => true
					),
					'order_id' => array(
						'required' => true
					)
				),
				'schema' => 'wc_airpay_get_order_details_schema',
			));
			register_rest_route($namespace,'/validate_order/', array(
				'methods' => 'POST',
				'callback' => 'wc_airpay_validate_order',
				'args' => array(
					'app_key' => array(
						'required' => true
					),
					'order_id' => array(
						'required' => true
					)
				),
			));
			register_rest_route($namespace,'/notify_order_status/', array( 
				'methods' => 'POST',
				'callback' => 'wc_airpay_notify_order_status',
				'args' => array(
					'app_key' => array(
						'required' => true
					),
					'order_id' => array(
						'required' => true
					),
					'order_status' => array(
						'required' => true
					),
					'payment_id' => array(
						'required' => true
					),
				),
			));
			register_rest_route($namespace,'/get_order_status/(?P<order_id>\d+)', array(
				'methods' => 'GET',
				'callback' => 'wc_airpay_get_order_status',
				'args' => array(
					'order_id' => array(
						'required' => true,
						'validate_callback' => function($param,$request, $key){
							return is_numeric($param);
						}
					)
				),
			));
		});

		function wc_airpay_get_order_details($request){
			global $woocommerce;

			$app_key = $request['app_key'];
			$order_id = $request['order_id'];
			$data = array('order_validity' => 200, 'order' => '');

			$woocommerce_airpay_settings = get_option('woocommerce_airpay_settings');
			if($woocommerce_airpay_settings['app_key'] != $app_key){
				return new WP_Error( 'wrong_app_key', 'Invalid app_key', array( 'status' => 400 ) );
			}
			$order = wc_get_order($order_id);
			if($order){
				$order_data = array();
				$order_data['app_id'] = $woocommerce_airpay_settings['app_id'];
				$order_data['order_id'] = $order_id;
				$order_data['currency'] = $order->get_currency();
				$order_data['payable_amount'] = $order->get_total()*100;
				$order_data['expiry_time'] = strtotime("+1 year");
                    if (sizeof($order->get_items()) > 0) {
                        foreach ($order->get_items() as $item) {
                            if ($item['qty']) {
                                $item_names[] = $item['name'] . ' x ' . $item['qty'];
                            }
                        }
                    }

                    $description = implode(",", $item_names);
				$order_data['item_name'] = $description;
				$data['order'] = $order_data;
			}else{
				$data['order_validity'] = 404;
			}

			return $data;
		}
		function wc_airpay_validate_order($request){
			global $woocommerce;

			$app_key = $request['app_key'];
			$order_id = $request['order_id'];
			$data = array('order_validity' => 200);

			$woocommerce_airpay_settings = get_option('woocommerce_airpay_settings');
			if($woocommerce_airpay_settings['app_key'] != $app_key){
				return new WP_Error( 'wrong_app_key', 'Invalid app_key', array( 'status' => 400 ) );
			}
			$order = wc_get_order($order_id);
			if(!$order){
				$data['order_validity'] = 404;
			}

			return $data;
		}
		function wc_airpay_notify_order_status($request){
			global $woocommerce;
			
			$app_key = $request['app_key'];
			$woocommerce_airpay_settings = get_option('woocommerce_airpay_settings');
			if($woocommerce_airpay_settings['app_key'] != $app_key){
				return new WP_Error( 'wrong_app_key', 'Invalid app_key', array( 'status' => 400 ) );
			}
			$order_id = $request['order_id'];
			$data['order_id'] = $order_id;
			$order = wc_get_order($order_id);
			if(!$order){
				return new WP_Error( 'order_not_found', 'Order not found', array( 'status' => 404 ) );
			}
			$order_status = $request['order_status'];
			$airpay_order_status_list = array(
				100,
				101,
				102,
				200,
				201,
				202,
				300,
				400,
				401,
				500,
				501,
			);
			if(!in_array($order_status, $airpay_order_status_list)){
				return new WP_Error( 'invalid_order_status', 'Order status is invalid', array( 'status' => 400 ) );
			}
			$payment_id = $request['payment_id'];
			switch($order_status){
				case 200: 
					//Money collected.
					//payment success
					//put payment_id in note
					$order->add_order_note(__("AirPay payment completed [Payment ID: $payment_id", 'woocommerce'));

					// Mark order complete
					$order->payment_complete();

					// Empty cart and clear session
					$woocommerce->cart->empty_cart();
					break;
				case 300:
					//Order has been canceled by customer. No money was collected.
					//Cancel order
					$order->update_status('cancelled', __('AirPay: Order has been canceled by customer.', 'woocommerce'));
					break;
				case 500:
					//Failed to collect money.
					$order->update_status('on-hold', __('AirPay error: Failed to collect money', 'woocommerce'));
					break;
				case 202:
					//Counter Service ------- No Counter Service for AirPay Payment Gateway
					//wait for payment on counter service 
					//no order status change.. wait for payment
					//put order note that customer select counter service
					break;
				case 401:
					//Order had expired while attempting to collect money. No money was collected.
				default:
					//nothing happen
					break;
			}
			return $data;
		}
		function wc_airpay_get_order_status($request){
			$order = wc_get_order($request['order_id']);
			if(!$order){
				return null;
			}
			$data['order_status'] = $order->get_status();

			return $data;
		}

		function wc_airpay_get_order_details_schema(){
			$schema = array(
				// This tells the spec of JSON Schema we are using which is draft 4.
				'$schema'              => 'http://json-schema.org/draft-04/schema#',
				// The title property marks the identity of the resource.
				'title'                => 'order_details',
				'type'                 => 'object',
				// In JSON Schema you can specify object properties in the properties attribute.
				'properties'           => array(
					'order_validity' => array(
						'description'  => esc_html__( 'Validity of the order in Merchant\'s system.', 'wc-airpay' ),
						'type'         => 'integer',
					),
					'order' => array(
						'description'  => esc_html__( 'Order details.', 'wc-airpay' ),
						'type'         => 'object',
						'properties'	=> array(
							'app_id'		=> array(
								'description'	=> esc_html__( 'The Merchant ID given by AirPay to the Merchant.', 'wc-airpay' ),
								'type'			=> 'integer',
							),
							'order_id'		=> array(
								'description'	=> esc_html__( 'The Order ID generated by the Merchant.', 'wc-airpay' ),
								'type'			=> 'string',
							),
							'currency'		=> array(
								'description'	=> esc_html__( 'Currency type. AirPay should use "THB"', 'wc-airpay' ),
								'type'			=> 'string',
							),
							'payable_amount'		=> array(
								'description'	=> esc_html__( 'Payment amount in the smallest currency unit. AirPay should use Thai Satang (1 THB = 100) Example: 123.50 THB = 12350', 'wc-airpay' ),
								'type'			=> 'integer',
							),
							'expiry_time'		=> array(
								'description'	=> esc_html__( 'Expiry time for this order. It is the Unix timestamp in seconds (number of seconds from 1970/01/01 00:00:00 GMT).', 'wc-airpay' ),
								'type'			=> 'integer',
							),
							'item_name'		=> array(
								'description'	=> esc_html__( 'Product\'s display name. This can be a brief product description to be displayed to use in AirPay App.', 'wc-airpay' ),
								'type'			=> 'string',
							),
							'item_image'		=> array(
								'description'	=> esc_html__( '(Optional) Product image URL. Only PNG and JPEG are supported. This image will not be shown in large size, so a smaller image (not larger than 150x150) is preferred.', 'wc-airpay' ),
								'type'			=> 'string',
							),
							'extra_data'		=> array(
								'description'	=> esc_html__( '(Optional) Additional information to tag with the order, JSON serialized.', 'wc-airpay' ),
								'type'			=> 'string',
							),
						),
					),
				),
			);

			return $schema;
		}

	}
}

/**
* Add airpay payment gateway into woocommerce payment gateway
*/
add_filter('woocommerce_payment_gateways', 'add_airpay_gateway');

function add_airpay_gateway($methods) {
	$methods[] = 'WC_Gateway_Airpay';
	return $methods;
}

function receipt_page_javascript(){
	global $woocommerce;
	?>
	<script language="javascript">
		jQuery(document).ready(function($){
			var check_order_status = function(){
				//get order status
				jQuery.get('<?=site_url('/wp-json/wc-airpay/v1/get_order_status/').$_GET['order']?>', function(data){
					//var obj = JSON.parse(data);
					if(data.order_status == 'processing' || data.order_status == 'completed'){
						//redirect to thankyou page
						window.location = '<?=WC_Payment_Gateway::get_return_url(wc_get_order($_GET['order']))?>';
					}
				});
			};
			var receipt_page_interval = setInterval(check_order_status,5000);
		});
	</script>
	<?php
}

add_action( 'wp_enqueue_scripts', 'airpay_site_scripts', 999 );
function airpay_site_scripts(){
	wp_localize_script( 'airpay_appjs', 'airpay_object',
		array(
			'api_nonce' => wp_create_nonce( 'wc_airpay' ),
			'api_url'   => site_url('/wp-json/wc-airpay/v1/')
		)
	);
}
